/**************************************
       
Chip type        : AT90S2313
Clock frequency  : 8,000000 MHz
Memory model     : Tiny
Internal RAM size: 128
External RAM size: 0
Data Stack size  : 32
***************************************/

#include <90s2313.h>

// Standard Input/Output functions
#include <stdio.h>

// Declare your global variables here

// Konstantendefinitionen
#define Slaveadresse 		0x90   	// Eigene Adresse
#define command_switch	 	0x01	// Kommando Lampen sofort ansteuern
#define command_sensor		0x02	// Personendetektor auslesen
#define command_ident		0x04	// Gert Identifizieren

char 	Kommando;			// Das empfangene Kommando
char	Datum;              // Empfangenes Datum
char 	Sensor;				// Sensordaten

unsigned int 	Seconds, ticker=0;			// Systemticker
                                     
// Timer 0 overflow interrupt service routine      
// Wird bei 10 MHz etwa alle 26ms angesprungen
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{
  ticker++; 
  
  if ( !(PINB & 32) )
   {                            
   	Sensor=1;
   }						// Sensor ist aktiv
        
  if (ticker==40)
   {
     Seconds++;  			// Eine Sekunde ist verstrichen
     ticker=0;
   }
}
                                                                                     
// Ganz kurze Pause machen, ca. 12.5 ms bei 8 Mhz
void delay (short int dauer)
{                         
	int inner,outer;		// Zhlvariablen
	
	for (outer=dauer;outer>0;outer--)  // Outer*0.7ms bei 8 Mhz
	 for (inner=255;inner>0;inner--);
	
} // end of delay 
          
               
// Sendet ein Acknowledge
void send_acknowledge (char typ)				// Typ=0 -> keine Zusatzinformation senden
{                     
	// Transmitter auf senden schalten
	PORTD|=64; // PD 6 auf High ziehen
	     
	// Kurze Pause fr den Bus
	delay (18);
	
    //  TXC auf Null setzen durch berschreiben mit 1
    USR|=64;
    
   	// Slaveadresse senden
   	UDR=Slaveadresse;

    // Warten, bis das Zeichen raus ist....
    delay (18);
                            
    // Auswerten, ob Zusatzinformationen gewnscht werden
    if (typ!=0)
     {
      Kommando=Kommando+(Sensor*16);	// Information in den oberen 4 Bits ausgeben
      Sensor=0;                         // Sensor wurde ausgelesen, zurcksetzen
     }
    
	//  TXC auf Null setzen durch berschreiben mit 1
    USR|=64;
    
	// Kommando hinterher schieben
	UDR=Kommando;
	
	// Hier noch warten, bis die Bytes raus sind
    // Warten, bis das Zeichen raus ist....
    delay (18);
     
	// Transmitter wieder abschalten
	PORTD&=255-64; // PD 6 auf Low ziehen
}

void main(void)
{
// Declare your local variables here

// Input/Output Ports initialization
// Port B
DDRB=0xFF;
PORTB=0x00;
  
// Port D
DDRD=64; 	// Alles Eingang, PD6 Ausgang, D5=Sensoreingang
PORTD=32;	// PD5 mit Pull-Up versehen 

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 9,766 kHz
// Mode: Output Compare
// OC0 output: Disconnected
// TCCR0=0x05; // 00000101 run
TCCR0=0x00; // 00000000 stopped
TCNT0=0x00;
   
// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 39,063 kHz
// Mode: 8 bit Pulse Width Modulation
// OC1 output: Inverted
// Noise Canceler: Off
// Input Capture on Falling Edge
TCCR1A=0xC1;
TCCR1B=0x04;
TCNT1H=0x00;
TCNT1L=0x00;
OCR1H=0x00;
OCR1AL=0xff;			// PWM-Startwert, entspricht 0%
             
// UART initialization
// Communication Parameters: 9 Data, 1 Stop, No Parity
// UART Receiver: On
// UART Transmitter: On
UCR=0x1C;
// UART Baud rate: 9600 @ 10 MHz
UBRR=0x40;               
           
// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x02;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
                 
// Timer 0 starten
TCCR0=0x03; 	// 00000011 run CK/64
            
// Global enable interrupts
#asm("sei")

while (1)
      {
      // Place your code here
  	   
      // Warten auf empfangenes Zeichen 
      while (!(USR & 0x80)) 
      {                                 
      }
      // Ist das 9. Datenbit gesetzt und die Slaveadresse gltig ?
      if ( (UDR==Slaveadresse) && ( UCR & 0x02) )
      {                                 
       // OK, wir sollten uns nun also angesprochen fhlen und den Arsch vom Sofa erheben.......
       
       // Warten auf Kommando
       while (!(USR & 0x80)) 
       {                               
       }	
       Kommando=UDR;
                                                            
       // Hier gelandet ist also der Slave korrekt angesprochen und das Kommando ist auch 
       // schon rbergekommen
       
       // send_acknowledge ();
                
       // Jetzt kann mit der Dekodierung begonnen werden
  
  	   switch (Kommando)
       {              
        
        case command_switch:           // Lampen direkt ansteuern
        	 {      
                // Warten auf das nchste Zeichen
     		   while (!(USR & 0x80)) 
       		    {                               
       		    }	
       		  Datum=UDR;      // Lampenwert
       		        		                     		  
       		  // Wert an Port B ausgeben  
       		  PORTB=Datum;
       		  
        	  send_acknowledge (0);		// Aknowledge rauschieben
        	  break;
        	 }
        
        case command_sensor:          		// Sensorsignal auswerten
        	 {                                                   
        	         	                               		  
        	  send_acknowledge (1);			// Aknowledge rauschieben mit der Sensorinformation
        	  break;
        	 }

        	       	
        case command_ident:				// Gert identifizieren
			 {                          
			  send_acknowledge (0);		// Aknowledge rauschieben
        	  break;
       		 }

   			 
       } // Von Switch 
       
       } // Von gltiger Slaveadresse mit 9. Datenbit gesetzt
       
      };
}
