/*

Project : RS485-Display

Chip type        : AT90S2313
Clock frequency  : 10,000000 MHz
Memory model     : Tiny
Internal RAM size: 128
External RAM size: 0
Data Stack size  : 32
***************************************/

#include <90s2313.h>

// Standard Input/Output functions
#include <stdio.h>

// Declare your global variables here     
                     
// Belegung der Hardware

// PB0 : D0 (out)
// ......
// PB7 : D7 (out)

// PD0 : RxD (In)
// PD1 : TxD (Out)
// PD2 : RS (Out)
// PD3 : n.c.
// PD4 : RE (Out)
// PD5 : EN (Out)
// PD6 : RW (Out)
 
// Konstantendefinitionen
#define Slaveadresse 		0xB0   	// Eigene Adresse, hier 176d	
#define command_get_char 	0x01	// Nachfolgendes Zeichen ausgeben	
#define command_get_com 	0x02	// Nachfolgendes Kommando ausfhren
#define command_ident		0x04	// Gert Identifizieren    
#define command_light		0x05	// Licht an oder aus	

char Kommando, Datum;		// Die empfangenen Daten, welche zum Display geschoben werden
char zeile;			// Welcher Controller soll angesprochen werden ?
char temp;			// Zwischenspeicher                 
char zaehler;	

// RS=High : Daten senden, PortD,2
// RS=Low  : Instruction senden

// RW=Low : Schreibzugriff auf Display, PortD,6
// RW=High : Lesezugriff auf Display      
       
// Erzeugt High-Impuls auf Enable
// 0 fr Zeile 1 & 3, 1 fr Zeile 2 & 4
void toggle_enable (void)
{             
 short int inner,outer; // Zhlvariablen
                   
 if (zeile==0) PORTB|=1;		// Enbale Controller 0 auf High
	else PORTB|=2;		// Enbale Controller 1 auf High 

 // ca. 1us Pause
 for (inner=2;inner>0;inner--) 
 { 
  outer=inner;
 };				
 
 PORTB&=(255-3); // Alle Enables auf Low
} // End of toggle_enable
                                                                         
// Ganz kurze Pause machen, ca. 10 ms bei 10 Mhz
void delay (short int dauer)
{                         
	int inner,outer;		// Zhlvariablen
	
	for (outer=dauer;outer>0;outer--)  // Outer*0.7ms bei 8 Mhz
	 for (inner=255;inner>0;inner--);
	
} // end of delay 

// Sendet ein Zeichen als Nibble zum Display (Zeichen bergeben in "Datum", Zeile in "Zeile"
void transmit (void)
 {                
 	temp=(Datum & 240);     // Zwischenspeichern, obere 4 Bits ausmaskieren
        PORTB=temp;     	 // Display-Kommando anlegen
        toggle_enable ();   	 // Und rein damit
        temp=(Datum & 15);	 // Zwischenspeichern, untere 4 Bits ausmaskieren
        temp=(temp<<4);		 // Bits nach oben schieben
        PORTB=temp;     	 // Display-Kommando anlegen
        toggle_enable ();   	 // Und rein damit
  }


// Display-Controller initialisieren (Nibble-Mode), je nach "Zeile"
void display_init (void)
{     
	PORTB=0x00; // Alles aus
	PORTD=0x00; // Alles aus
	
	PORTB=48;    // 00110000
	toggle_enable ();
  	delay (10);
    	toggle_enable ();
  	delay (10);
    	toggle_enable ();
  	delay (10);
        
        PORTB=32;    // 00100000 Umschalten auf Nibble-Mode
	toggle_enable ();
                                                           
 	// Ab jetzt nur noch Nibble-Kommunikation, obere 4 Bits zuerst
                                   
    	// Font einstellen 5*11 dots
    	Datum=44;
    	transmit ();
	delay (10);
   
   	// Display off, cursor off, blink off                             
   	Datum=8;
   	transmit ();
	delay (10);
   
   	// Clear Screen, Home
    Datum=1;
    transmit ();
	delay (10);
                        
	// Increment Cursor, dont shift
	Datum=6;
	transmit ();
    	delay (10);
         
   	// Display wieder ein, Cursor aus
	Datum=12;
	transmit ();
	delay (10);
   
   
}//end of display_init
                                         
// Sendet ein Acknowledge
void send_acknowledge (void)
{                     
	// Transmitter auf senden schalten
	PORTD|=16; // PD 4 auf High ziehen
	     
	// Kurze Pause fr den Bus
	delay (18);
	
    //  TXC auf Null setzen durch berschreiben mit 1
    USR|=64;
    
   	// Slaveadresse senden
   	UDR=Slaveadresse;

    // Warten, bis das Zeichen raus ist....
    delay (18);
    
	//  TXC auf Null setzen durch berschreiben mit 1
    USR|=64;
    
	// Kommando hinterher schieben
	UDR=Kommando;
	
	// Hier noch warten, bis die Bytes raus sind
    // Warten, bis das Zeichen raus ist....
    delay (18);
     
	// Transmitter wieder abschalten
	PORTD&=239; // PD 4 auf Low ziehen
	 
}
                     
//EEPROM  read, returns data byte       
char read_eeprom (char address)
{  
   while (EECR & 0x02);       //wait for EEWE==0
   EEAR = address;            //write address to (EEAR)
   EECR = 0x01;               //set EERE=1 (EECR.0)
   return (EEDR);
}//end function
       
//---------------------------------------------
// Hauptprogramm                              
//---------------------------------------------
void main(void)
{
// Declare your local variables here

// Input/Output Ports initialization
// Port B
DDRB=0xFF;    // Alles Output
PORTB=0x00;

// Port D
DDRD=0x7C; // 01111100, RXD, TXD ausgenommen
PORTD=0x00;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: Timer 0 Stopped
// Mode: Output Compare
// OC0 output: Disconnected
TCCR0=0x00;
TCNT0=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer 1 Stopped
// Mode: Output Compare
// OC1 output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
OCR1H=0x00;
OCR1L=0x00;

// UART initialization
// Communication Parameters: 9 Data, 1 Stop, No Parity
// UART Receiver: On
// UART Transmitter: On
UCR=0x1C;
// UART Baud rate: 9600 @ 10 MHz
UBRR=0x40;  

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
                  
// Display initialisieren
zeile=0;
display_init ();	// Ungerade Zeilen
zeile=1;
display_init ();       // Gerade Zeilen
                                     
 
// Starttext ausgeben
zeile=0;			// Controller 0 ansprechen

// Cursor auf Anfg zweite Zeile setzen
PORTD&=0xFB;		// PortD,2 (RS) auf Low ziehen
Datum=(64+128);
transmit ();		// Und raus damit

PORTD|=4;			// PortD,2 (RS) auf High ziehen
for (zaehler=1;zaehler<30;zaehler++)
 {
  Datum=read_eeprom (zaehler);
  // Datum=65+zaehler;
  delay (10);	
  transmit ();
 }


while (1)
      {
      // Warten auf empfangenes Zeichen 
      while (!(USR & 0x80)) 
      {                                 
      }
      // Ist das 9. Datenbit gesetzt und die Slaveadresse gltig ?
      if ( (UDR==Slaveadresse) && ( UCR & 0x02) )
      {                                 
       // OK, wir sollten uns nun also angesprochen fhlen und den Arsch vom Sofa erheben.......
      
       // Als nchstes werden noch 2 Bytes folgen, die die Daten enthalten
       
       // Warten auf empfangenes Zeichen 
      while (!(USR & 0x80)) 
      {                               
      }	
      Kommando=UDR;
       
      // Hier gelandet ist also der Slave korrekt angesprochen und das Kommando ist auch 
       // schon rbergekommen
       
       // send_acknowledge ();
                
       // Jetzt kann mit der Dekodierung begonnen werden
       switch (Kommando)
       {              
          
        case command_ident:             // Slave identifizieren
        	 {      
        	  send_acknowledge ();		// Aknowledge rauschieben
        	  break;
        	 }
      
         case command_get_char:			// Nachfolgendes Zeichen ausgeben	
        	 {            
        	 
        	  // Warten auf das nchste Zeichen, welcher Controller soll angesprochen werden
     		   while (!(USR & 0x80)) 
       		    {                               
       		    }	
               	   zeile=UDR; 
        	   
        	 // Die Zeichen kommen jetzt als Null-terminierter String rber
        	 Datum=1;
        	 while (Datum!=0) // Abbruchkriterium setzen
        	 {
                   // Warten auf das nchste Zeichen
     		   while (!(USR & 0x80)) 
       		    {                               
       		    }	
                   Datum=UDR; 
                  
              	 if (Datum!=0)  // Auszugebendes Zeichen ?
              	 {                               
              	  PORTD|=4;			// PortD,2 (RS) auf High ziehen
 		  transmit ();			// Und raus damit           	 
              	 } // Von if (datum!=0)	
			              
        	 } // Von While
        	 
        	  send_acknowledge ();		// Aknowledge rauschieben
        	  break;
        	 }
	     
	     case command_get_com:			// Nachfolgendes Kommando ausfhren
        	 {
        	  // Warten auf das nchste Zeichen, welcher Controller soll angesprochen werden
     		   while (!(USR & 0x80)) 
       		    {                               
       		    }	
               	   zeile=UDR; 
                  
                  // Warten auf das nchste Zeichen, das eigentliche Kommando
     		   while (!(USR & 0x80)) 
       		    {                               
       		    }	
                   Datum=UDR; 
                  
                            
              	// Das Kommando ist rber und kann zum Display geschoben werden 
              	PORTD&=0xFB;		// PortD,2 (RS) auf Low ziehen
                transmit ();		// Und raus damit
                
              	send_acknowledge ();	 // Aknowledge rauschieben
        	 break;
              
        	 }
        	 
		case command_light:			// Licht ein- und ausschalten
        	 {
        	  // Warten auf das nchste Zeichen
     		   while (!(USR & 0x80)) 
       		    {                               
       		    }	
              Datum=UDR; 
                            
              // Ist Datum=0, dann Licht aus, sonst Licht ein
              if (Datum==0) 
              {           // Licht aus
                PORTD&=247;  // Bit 3 auf Null
              }  else
              
              {           // Licht ein        
              	PORTD|=8;  // Bit 3 auf Eins
              };
              
              send_acknowledge ();		// Aknowledge rauschieben
        	  break;
        	 }
        	         	 
       } // Von Switch 
     
     }
     
     } // Von While (1)
}
